'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Node = function () {
  function Node(value) {
    _classCallCheck(this, Node);

    this.value = value;
    this.left = null;
    this.right = null;
  }

  /**
   * Get total number of elements in tree / subtrees
   * @return {number}
   */


  _createClass(Node, [{
    key: 'size',
    value: function size() {
      var calculateSize = function calculateSize(node) {
        if (node === null) {
          return 0;
        }

        return 1 + calculateSize(node.left) + calculateSize(node.right);
      };

      return calculateSize(this);
    }

    /**
     * @return {number}
     */

  }, {
    key: 'height',
    value: function height() {
      var calculateHeight = function calculateHeight(node) {
        if (node === null) {
          return 0;
        }

        return Math.max(1 + calculateHeight(node.left), 1 + calculateHeight(node.right));
      };

      return calculateHeight(this);
    }

    /**
     * @return {boolean}
     */

  }, {
    key: 'isLeaf',
    value: function isLeaf() {
      return !this.left && !this.right;
    }
  }]);

  return Node;
}();

/**
 * levelOrder()
 */


var BST = function () {
  function BST() {
    _classCallCheck(this, BST);

    this.root = null;
  }

  /**
   * @return {number}
   */


  _createClass(BST, [{
    key: 'size',
    value: function size() {
      return this.root.size();
    }

    /**
     * @return {number}
     */

  }, {
    key: 'height',
    value: function height() {
      return this.root.height();
    }

    /**
     * @param {*} value
     */

  }, {
    key: 'add',
    value: function add(value) {
      var node = new Node(value);

      if (this.root === null) {
        this.root = node;
      } else {
        var searchTree = function searchTree(node, root) {
          if (node.value <= root.value) {
            if (root.left === null) {
              root.left = node;
            } else {
              searchTree(node, root.left);
            }
          } else if (node.value > root.value) {
            if (root.right === null) {
              root.right = node;
            } else {
              searchTree(node, root.right);
            }
          }
        };

        searchTree(node, this.root);
      }
    }

    /**
     * @param  {*} value
     * @return {(Node | boolean)}
     */

  }, {
    key: 'search',
    value: function search(value) {
      if (this.root === null) {
        return false;
      }

      var current = this.root;

      while (current !== null) {
        if (value === current.value) {
          return current;
        }

        if (value < current.value) {
          current = current.left;
        }

        if (value > current.value) {
          current = current.right;
        }
      }

      return false;
    }

    /**
     * @return {(boolean | *)}
     */

  }, {
    key: 'findMin',
    value: function findMin() {
      if (this.root === null) {
        return false;
      }

      var current = this.root;

      while (current.left !== null) {
        current = current.left;
      }

      return current.value;
    }

    /**
     * @return {(boolean | *)}
     */

  }, {
    key: 'findMax',
    value: function findMax() {
      if (this.root === null) {
        return false;
      }

      var current = this.root;

      while (current.right !== null) {
        current = current.right;
      }

      return current.value;
    }

    /**
     * @param  {*} value
     * @return {Node | null}
     */

  }, {
    key: 'delete',
    value: function _delete(value) {
      var deleteNode = function deleteNode(value, node) {
        if (node === null) {
          return null;
        }

        if (node.value === value) {
          // node has no children
          if (node.isLeaf()) {
            return null;
          }

          // node has one child
          if (node.left === null || node.right === null) {
            return node.left === null ? node.right : node.left;
          }

          // node has two children
          var tmp = node.right;
          while (tmp.left !== null) {
            tmp = tmp.left;
          }

          node.value = tmp.value;
          node.right = deleteNode(tmp.value, node.right);
        } else if (value < node.value) {
          node.left = deleteNode(value, node.left);
        } else {
          node.right = deleteNode(value, node.right);
        }

        return node;
      };

      this.root = deleteNode(value, this.root);
    }

    /**
     * @return {Array}
     */

  }, {
    key: 'preOrder',
    value: function preOrder() {
      var preOrderTraversal = function preOrderTraversal(node, result) {
        if (node === null) {
          return;
        }

        result.push(node.value);

        if (node.left) {
          result.concat(preOrderTraversal(node.left, result));
        }

        if (node.right) {
          result.concat(preOrderTraversal(node.right, result));
        }

        return result;
      };

      return preOrderTraversal(this.root, []);
    }

    /**
     * @return {Array}
     */

  }, {
    key: 'inOrder',
    value: function inOrder() {
      var inOrderTraversal = function inOrderTraversal(node, result) {
        if (node === null) {
          return;
        }

        if (node.left) {
          result.concat(inOrderTraversal(node.left, result));
        }

        result.push(node.value);

        if (node.right) {
          result.concat(inOrderTraversal(node.right, result));
        }

        return result;
      };

      return inOrderTraversal(this.root, []);
    }

    /**
     * @return {Array}
     */

  }, {
    key: 'postOrder',
    value: function postOrder() {
      var postOrderTraversal = function postOrderTraversal(node, result) {
        if (node === null) {
          return;
        }

        if (node.left) {
          result.concat(postOrderTraversal(node.left, result));
        }

        if (node.right) {
          result.concat(postOrderTraversal(node.right, result));
        }

        result.push(node.value);

        return result;
      };

      return postOrderTraversal(this.root, []);
    }

    /**
     * @return {Array}
     */

  }, {
    key: 'levelOrder',
    value: function levelOrder() {
      var getLevel = function getLevel(root, level, result) {
        if (root === null) {
          return;
        }

        if (level === 1) {
          return result.push(root.value);
        }

        getLevel(root.left, level - 1, result);
        getLevel(root.right, level - 1, result);
      };

      var result = [];

      for (var i = 1; i <= this.root.height(); i++) {
        getLevel(this.root, i, result);
      }

      return result;
    }

    /**
     * @param  {Node} node
     */

  }, {
    key: 'invert',
    value: function invert(node) {
      var invertSubtree = function invertSubtree(node) {
        if (node === null) {
          return;
        }

        var tmp = node.left;
        node.left = node.right;
        node.right = tmp;

        invertSubtree(node.left);
        invertSubtree(node.right);
      };

      invertSubtree(this.root);
    }
  }]);

  return BST;
}();

exports.default = BST;
;