'use strict';

/**
 * @param  {number} i
 * @param  {number} j
 * @param  {Array<number>} array
 */

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function swap(i, j, array) {
  var tmp = array[i];
  array[i] = array[j];
  array[j] = tmp;
}

/**
 * Min heap implementation using an array
 */

var MinHeap = function () {
  function MinHeap() {
    _classCallCheck(this, MinHeap);

    this.values = [];
  }

  /**
   * @param  {number} index
   * @return {number}
   */


  _createClass(MinHeap, [{
    key: 'getLeftChildIndex',
    value: function getLeftChildIndex(index) {
      return index * 2 + 1;
    }

    /**
     * @param  {number} index
     * @return {number}
     */

  }, {
    key: 'getRightChildIndex',
    value: function getRightChildIndex(index) {
      return index * 2 + 2;
    }

    /**
     * @param  {number} index
     * @return {number}
     */

  }, {
    key: 'getParentIndex',
    value: function getParentIndex(index) {
      return Math.ceil((index - 2) / 2);
    }

    /**
     * @param  {number} index
     * @return {number}
     */

  }, {
    key: 'getLeftChild',
    value: function getLeftChild(index) {
      this.checkIfEmpty();

      return this.values[this.getLeftChildIndex(index)];
    }

    /**
     * @param  {number} index
     * @return {number}
     */

  }, {
    key: 'getRightChild',
    value: function getRightChild(index) {
      this.checkIfEmpty();

      return this.values[this.getRightChildIndex(index)];
    }

    /**
     * @param  {number} index
     * @return {number}
     */

  }, {
    key: 'getParent',
    value: function getParent(index) {
      this.checkIfEmpty();

      return this.values[this.getParentIndex(index)];
    }

    /**
     * @param  {number}  index
     * @return {Boolean}
     */

  }, {
    key: 'hasLeftChild',
    value: function hasLeftChild(index) {
      return this.getLeftChildIndex(index) < this.values.length;
    }

    /**
     * @param  {number}  index
     * @return {Boolean}
     */

  }, {
    key: 'hasRightChild',
    value: function hasRightChild(index) {
      return this.getRightChildIndex(index) < this.values.length;
    }

    /**
     * @param  {number} index
     */

  }, {
    key: 'add',
    value: function add(value) {
      this.values.push(value);

      if (this.values.length > 1) {
        this.heapifyUp();
      }
    }

    /**
     * @return {number}
     */

  }, {
    key: 'peek',
    value: function peek() {
      this.checkIfEmpty();

      return this.values[0];
    }
  }, {
    key: 'heapifyUp',
    value: function heapifyUp() {
      var currentIndex = this.values.length - 1;

      while (currentIndex > 0 && this.values[currentIndex] < this.getParent(currentIndex)) {
        var parentIndex = this.getParentIndex(currentIndex);

        swap(currentIndex, parentIndex, this.values);
        currentIndex = parentIndex;
      }
    }

    /**
     * @return {number}
     */

  }, {
    key: 'poll',
    value: function poll() {
      this.checkIfEmpty();

      swap(0, this.values.length - 1, this.values);
      var minNum = this.values.pop();

      if (this.values.length > 1) {
        this.heapifyDown();
      }

      return minNum;
    }
  }, {
    key: 'heapifyDown',
    value: function heapifyDown() {
      var currentIndex = 0;

      while (this.hasLeftChild(currentIndex)) {
        var smallestChildIndex = this.getLeftChildIndex(currentIndex);

        if (this.hasRightChild(currentIndex) && this.getRightChild(currentIndex) < this.getLeftChild(currentIndex)) {
          smallestChildIndex = this.getRightChildIndex(currentIndex);
        }

        if (this.values[currentIndex] < this.values[smallestChildIndex]) {
          break;
        }

        swap(smallestChildIndex, currentIndex, this.values);
        currentIndex = smallestChildIndex;
      }
    }
  }, {
    key: 'checkIfEmpty',
    value: function checkIfEmpty() {
      if (!this.values.length) {
        throw Error('Heap is empty!');
      }
    }
  }]);

  return MinHeap;
}();

exports.default = MinHeap;
;